@@ -2,6 +2,20 @@
 ###
 ### Copyright 1997-2003  Jose C. Pinheiro <Jose.Pinheiro@pharma.novartis.com>,
 ###                      Douglas M. Bates <bates@stat.wisc.edu>
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+#
 
 lme <-
   ## fits general linear mixed effects model by maximum likelihood, or
@@ -15,7 +29,7 @@ lme <-
 	   method = c("REML", "ML"),
 	   na.action = na.fail,
 	   control = list(),
-           contrasts = NULL)
+           contrasts = NULL, keep.data = TRUE)
   UseMethod("lme")
 
 lme.groupedData <-
@@ -28,7 +42,7 @@ lme.groupedData <-
 	   method = c("REML", "ML"),
 	   na.action = na.fail,
 	   control = list(),
-           contrasts = NULL)
+           contrasts = NULL, keep.data = TRUE)
 {
   args <- as.list(match.call())[-1]
   names(args)[1] <- "data"
@@ -47,7 +61,7 @@ lme.lmList <-
 	   method = c("REML", "ML"),
 	   na.action = na.fail,
 	   control = list(),
-           contrasts = NULL)
+           contrasts = NULL, keep.data = TRUE)
 {
   if (length(grpForm <- getGroupsFormula(fixed, asList = TRUE)) > 1) {
     stop("Can only fit lmList objects with single grouping variable")
@@ -66,7 +80,7 @@ lme.lmList <-
   this.call[names(last.call)] <- last.call
   this.call$fixed <-
     as.vector(eval(parse(text=paste(deparse(getResponseFormula(fixed)[[2]]),
-                   deparse(getCovariateFormula(fixed)[[2]]), sep="~"))))
+                   c_deparse(getCovariateFormula(fixed)[[2]]), sep="~"))))
   if (missing(random)) {
     random <- eval(as.call(this.call[["fixed"]][-2]))
   }
@@ -118,9 +132,11 @@ lme.formula <-
 	   method = c("REML", "ML"),
 	   na.action = na.fail,
 	   control = list(),
-           contrasts = NULL)
+           contrasts = NULL,
+           keep.data = TRUE)
 {
   Call <- match.call()
+  miss.data <- missing(data) || !is.data.frame(data)
 
   ## control parameters
   controlvals <- lmeControl()
@@ -186,7 +202,7 @@ lme.formula <-
                         "the former with the latter."))
           attr(correlation, "formula") <-
             eval(parse(text = paste("~",
-                    deparse(getCovariateFormula(formula(correlation))[[2]]),
+                    c_deparse(getCovariateFormula(formula(correlation))[[2]]),
                          "|", deparse(groups[[2]]))))
         }
       } else {
@@ -199,7 +215,7 @@ lme.formula <-
       ## using the same grouping as in random
       attr(correlation, "formula") <-
         eval(parse(text = paste("~",
-		     deparse(getCovariateFormula(formula(correlation))[[2]]),
+		     c_deparse(getCovariateFormula(formula(correlation))[[2]]),
 		     "|", deparse(groups[[2]]))))
       corQ <- lmeQ <- 1
     }
@@ -258,6 +274,7 @@ lme.formula <-
   ## keeping the contrasts for later use in predict
   contr <- attr(Z, "contr")
   X <- model.frame(fixed, dataMix)
+  Terms <- attr(X, "terms")
   auxContr <- lapply(X, function(el)
 		     if (inherits(el, "factor") &&
                          length(levels(el)) > 1) contrasts(el))
@@ -281,7 +298,7 @@ lme.formula <-
   }
   ## degrees of freedom for testing fixed effects
   fixDF <- getFixDF(X, grps, attr(lmeSt, "conLin")$dims$ngrps,
-                    terms = terms(fixed))
+                    terms = Terms)
   ## initialization
   lmeSt <- Initialize(lmeSt, dataMix, grps, control = controlvals)
   parMap <- attr(lmeSt, "pmap")
@@ -299,18 +316,38 @@ lme.formula <-
   numIter <- 0
   repeat {
     oldPars <- coef(lmeSt)
-    aNlm <- optim(fn = function(lmePars) -logLik(lmeSt, lmePars),
-                  par = c(coef(lmeSt)),
-                  method = "BFGS",
-                  control = list(trace = controlvals$msVerbose,
-                  reltol = if(numIter == 0) controlvals$msTol
-                           else 100*.Machine$double.eps,
-                  maxit = controlvals$msMaxIter))
+    optRes <- if (controlvals$opt == "nlminb") {
+        nlminb(c(coef(lmeSt)),
+               function(lmePars) -logLik(lmeSt, lmePars),
+               control = list(iter.max = controlvals$msMaxIter,
+               eval.max = controlvals$msMaxEval,
+               trace = controlvals$msVerbose))
+    } else {
+        optim(c(coef(lmeSt)),
+              function(lmePars) -logLik(lmeSt, lmePars),
+              control = list(trace = controlvals$msVerbose,
+              maxit = controlvals$msMaxIter,
+              reltol = if(numIter == 0) controlvals$msTol
+              else 100*.Machine$double.eps),
+              method = controlvals$optimMethod)
+    }
     numIter0 <- NULL
-    coef(lmeSt) <- aNlm$par
+    coef(lmeSt) <- optRes$par
     attr(lmeSt, "lmeFit") <- MEestimate(lmeSt, grps)
     ## checking if any updating is needed
-    if (!needUpdate(lmeSt)) break
+    if (!needUpdate(lmeSt)) {
+	if (optRes$convergence) {
+	    msg <- paste(controlvals$opt, " problem, convergence error code = ",
+			 optRes$convergence, "\n  message = ", optRes$message,
+			 sep='')
+	    if(!controlvals$returnObject)
+		stop(msg)
+	    else
+		warning(msg)
+	}
+	break
+    }
+
     ## updating the fit information
     numIter <- numIter + 1
     lmeSt <- update(lmeSt, dataMix)
@@ -319,18 +356,25 @@ lme.formula <-
     conv <- abs((oldPars - aConv)/ifelse(aConv == 0, 1, aConv))
     aConv <- NULL
     for(i in names(lmeSt)) {
-      if (any(parMap[,i])) {
-	aConv <- c(aConv, max(conv[parMap[,i]]))
-	names(aConv)[length(aConv)] <- i
-      }
+	if (any(parMap[,i])) {
+	    aConv <- c(aConv, max(conv[parMap[,i]]))
+	    names(aConv)[length(aConv)] <- i
+	}
     }
     if (max(aConv) <= controlvals$tolerance) {
-      break
+	break
     }
     if (numIter > controlvals$maxIter) {
-      stop("Maximum number of iterations reached without convergence.")
+	msg <- paste("Maximum number of iterations",
+		     "(lmeControl(maxIter)) reached without convergence.")
+	if (controlvals$returnObject) {
+	    warning(msg)
+	    break
+	} else
+	    stop(msg)
     }
-  }
+
+  } ## end{repeat}
 
   ## wrapping up
   lmeFit <- attr(lmeSt, "lmeFit")
@@ -342,12 +386,10 @@ lme.formula <-
   ## fitted.values and residuals (in original order)
   ##
   Fitted <- fitted(lmeSt, level = 0:Q,
-		   conLin = if (decomp) {
-		     oldConLin
-		   } else {
-		     attr(lmeSt, "conLin")
-		   })[revOrder, , drop = FALSE]
+		   conLin = if (decomp) oldConLin else attr(lmeSt, "conLin"))[
+		   revOrder, , drop = FALSE]
   Resid <- y[revOrder] - Fitted
+  rownames(Resid) <- rownames(Fitted) <- origOrder
   attr(Resid, "std") <- lmeFit$sigma/(varWeights(lmeSt)[revOrder])
   ## putting groups back in original order
   grps <- grps[revOrder, , drop = FALSE]
@@ -389,10 +431,13 @@ lme.formula <-
 		   else numIter0,
 		 groups = grps,
 		 call = Call,
+                 terms = Terms,
 		 method = method,
 		 fitted = Fitted,
 		 residuals = Resid,
-                 fixDF = fixDF)
+                 fixDF = fixDF,
+                 na.action = attr(dataMix, "na.action"))
+  if (keep.data && !miss.data) estOut$data <- data
   if (inherits(data, "groupedData")) {
     ## saving labels and units for plots
     attr(estOut, "units") <- attr(data, "units")
@@ -440,14 +485,13 @@ getFixDF <-
   if (any(notIntX <- !apply(X, 2, const))) {
       ## percentage of groups for which columns of X are inner
       innP <- array(c(rep(1, p),
-                      .C("inner_perc_table",
+                      .C(inner_perc_table,
                          as.double(X),
                          as.integer(unlist(grps)),
                          as.integer(p),
                          as.integer(Q),
                          as.integer(N),
-                         val = double(p * Q),
-                         PACKAGE = "nlme")[["val"]]), c(p, Qp1),
+                         val = double(p * Q))[["val"]]), c(p, Qp1),
                     list(namX, stratNam))
     ## strata in which columns of X are estimated
     ## ignoring fractional inner percentages for now
@@ -520,14 +564,13 @@ lmeApVar <-
 	  NULL
 	}
       }
-      val <- .C("mixed_loglik",
+      val <- .C(mixed_loglik,
 		as.double(conLin$Xy),
 		as.integer(unlist(dims)),
 		as.double(sigma * unlist(pdFactor(solve(object$reStruct)))),
 		as.integer(settings),
 		logLik = double(1),
-		lRSS = double(1),
-		PACKAGE = "nlme")[c("logLik", "lRSS")]
+		lRSS = double(1))[c("logLik", "lRSS")]
       aux <- (exp(val[["lRSS"]])/sigma)^2
       conLin[["logLik"]] + val[["logLik"]] + (N * log(aux) - aux)/2
     }
@@ -538,13 +581,13 @@ lmeApVar <-
   conLin[["logLik"]] <- 0               # making sure
   sig2 <- sigma * sigma
   reSt <- lmeSt[["reStruct"]]
-  for(i in seq(along = reSt)) {
+  for(i in seq_along(reSt)) {
     matrix(reSt[[i]]) <- as.double(sig2) * pdMatrix(reSt[[i]])
     if (inherits(reSt[[i]], "pdSymm") && natural) {
       reSt[[i]] <- pdNatural(reSt[[i]])
     }
     if (inherits(reSt[[i]], "pdBlocked") && natural) {
-      for(j in seq(along = reSt[[i]])) {
+      for(j in seq_along(reSt[[i]])) {
         if (inherits(reSt[[i]][[j]], "pdSymm")) {
           reSt[[i]][[j]] <- pdNatural(reSt[[i]][[j]])
         }
@@ -586,10 +629,9 @@ MEdecomp <-
     ## no pint in doing the decomposition
     return(conLin)
   }
-  dc <- array(.C("mixed_decomp",
+  dc <- array(.C(mixed_decomp,
 		 as.double(conLin$Xy),
-		 as.integer(unlist(dims)),
-		 PACKAGE = "nlme")[[1]],
+		 as.integer(unlist(dims)))[[1]],
 	      c(dims$StrRows, dims$ZXcols))
   dims$ZXrows <- dims$StrRows
   dims$ZXoff <- dims$DecOff
@@ -608,7 +650,7 @@ MEEM <-
     pdCl <- attr(object, "settings")[-(1:3)]
     pdCl[pdCl == -1] <- 0
     precvec <- unlist(pdFactor(object))
-    zz <- .C("mixed_EM",
+    zz <- .C(mixed_EM,
 	     as.double(conLin$Xy),
 	     as.integer(unlist(dd)),
 	     precvec = as.double(precvec),
@@ -617,11 +659,10 @@ MEEM <-
 	     as.integer(attr(object, "settings")[1]),
 	     double(1),
 	     double(length(precvec)),
-	     double(1),
-	     PACKAGE = "nlme")[["precvec"]]
+	     double(1))[["precvec"]]
     Prec <- vector("list", length(object))
     names(Prec) <- names(object)
-    for (i in seq(along = object)) {
+    for (i in seq_along(object)) {
       len <- dd$qvec[i]^2
       matrix(object[[i]]) <-
         crossprod(matrix(zz[1:len + dd$DmOff[i]], ncol = dd$qvec[i]))
@@ -638,15 +679,14 @@ MEestimate <-
   REML <- attr(object$reStruct, "settings")[1]
   Q <- dd$Q
   rConLin <- recalc(object, conLin)
-  zz <- .C("mixed_estimate",
+  zz <- .C(mixed_estimate,
 	   as.double(rConLin$Xy),
 	   as.integer(unlist(dd)),
 	   as.double(unlist(pdFactor(object$reStruct))),
 	   as.integer(REML),
 	   double(1),
 	   estimates = double(dd$StrRows * dd$ZXcols),
-	   as.logical(FALSE),
-	   PACKAGE = "nlme")[["estimates"]]
+	   as.logical(FALSE))[["estimates"]]
   estimates <- array(zz, c(dd$StrRows, dd$ZXcols))
   resp <- estimates[ , dd$ZXcols]
   reSt <- object$reStruct
@@ -654,7 +694,7 @@ MEestimate <-
   val <- vector(mode = "list", length = Q)
   names(val) <- nam
   start <- dd$StrRows * c(0, cumsum(nc))
-  for (i in seq(along = reSt)) {
+  for (i in seq_along(reSt)) {
     val[[i]] <-
       matrix(resp[as.vector(outer(1:(nc[i]), dd$SToff[[i]] - start[i], "+"))],
 	     ncol = nc[i], byrow = TRUE,
@@ -687,7 +727,7 @@ MEdims <-
   {
     pop <- function(x) x[-length(x)]
     cstart <- c(0, cumsum(N * ncols))
-    for (i in seq(along = lstrow)) {
+    for (i in seq_along(lstrow)) {
       lstrow[[i]] <- cstart[i] +
         if (triangle) {
           lstrow[[i]] - ncols[i]        # storage offsets style
@@ -713,7 +753,7 @@ MEdims <-
     isLast[, Q + 1 - i] <- c(0 != diff(as.integer(groups[[i]])), TRUE)
   }
   isLast[N,  ] <- TRUE
-  lastRow <- apply(isLast, 2, function(x) seq(along = x)[x])
+  lastRow <- apply(isLast, 2, function(x) seq_along(x)[x])
   if(!is.list(lastRow)) {
     nm <- names(lastRow)
     lastRow <- as.list(lastRow)
@@ -1175,21 +1215,24 @@ fitted.lme <-
   } else {				# assuming integers
     level <- 1 + level
   }
-  val <- val[, level]
+  val2 <- napredict(object$na.action, val[, level])
   if (length(level) == 1) {
+    grp.nm <- row.names(object[["groups"]])
     grps <- as.character(object[["groups"]][, max(c(1, level - 1))])
     if (asList) {
       val <- as.list(split(val, ordered(grps, levels = unique(grps))))
     } else {
-      names(val) <- grps
+      grp.nm <- row.names(object[["groups"]])
+      val <- val2
+      names(val) <- grps[match(names(val), grp.nm)]
     }
     lab <- "Fitted values"
     if (!is.null(aux <- attr(object, "units")$y)) {
       lab <- paste(lab, aux)
     }
     attr(val, "label") <- lab
-  }
-  val
+    val
+  } else val2
 }
 
 formula.lme <- function(x, ...) eval(x$call$fixed)
@@ -1280,12 +1323,12 @@ intervals.lme <-
     origInt <- origInt[-nP,, drop = FALSE]
 
     if (attr(aV, "natural")) {          # convert any pdSymm's to pdNatural's
-      for(i in seq(along = lmeSt$reStruct)) {
+      for(i in seq_along(lmeSt$reStruct)) {
 	if (inherits(lmeSt$reStruct[[i]], "pdSymm")) {
 	  lmeSt$reStruct[[i]] <- pdNatural(lmeSt$reStruct[[i]])
 	} else {
           if (inherits(lmeSt$reStruct[[i]], "pdBlocked")) {
-            for(j in seq(along = lmeSt$reStruct[[i]])) {
+            for(j in seq_along(lmeSt$reStruct[[i]])) {
               if (inherits(lmeSt$reStruct[[i]][[j]], "pdSymm")) {
                 lmeSt$reStruct[[i]][[j]] <- pdNatural(lmeSt$reStruct[[i]][[j]])
               }
@@ -1295,7 +1338,8 @@ intervals.lme <-
       }
     }
     rownames(origInt) <-           # re-express names if necessary
-      namP <- names(coef(lmeSt, unconstrained = FALSE))
+      ## namP <-
+          names(coef(lmeSt, unconstrained = FALSE))
     for(i in 1:3) {                     # re-express intervals in constrained pars
       coef(lmeSt) <- origInt[,i]
       origInt[,i] <- coef(lmeSt, unconstrained = FALSE)
@@ -1308,7 +1352,7 @@ intervals.lme <-
                natInt[[i]] <-
                natInt[[i]] <-
                  rev(as.matrix( split( as.data.frame( natInt[[i]] ),
-                                      rep( seq(along = plen), plen ))))
+                                      rep( seq_along(plen), plen ))))
                names(natInt[[i]]) <- rev(names(plen))
                for (j in names(plen)) {
                  dimnames(natInt[[i]][[j]])[[1]] <-
@@ -1340,7 +1384,7 @@ logLik.lme <-
 {
   p <- object$dims$ncol[object$dims$Q + 1]
   N <- object$dims$N
-  Np <- N - p
+##  Np <- N - p
   estM <- object$method
   if (missing(REML)) REML <- estM == "REML"
   val <- object[["logLik"]]
@@ -1459,7 +1503,7 @@ pairs.lme <-
        }
 	       aux <- t(as.matrix(ranef(object, level = level)))
 	       aux <- as.logical(apply(
-	(solve(t(pdMatrix(object$modelStruct$reStruct, fact = TRUE)[[level]]),
+	(solve(t(pdMatrix(object$modelStruct$reStruct, factor = TRUE)[[level]]),
 		 aux)/object$sigma)^2, 2, sum) > qchisq(1 - id, dim(aux)[1]))
 	       aux
 	     },
@@ -1520,7 +1564,7 @@ pairs.lme <-
   assign("idLabels", as.character(idLabels))#, where = 1)
   #assign("grid", grid, where = 1)
   ## adding to args list
-  args <- c(list(formula = argForm, data = argData), args)
+  args <- c(list(argForm, data = argData), args)
   if (is.null(args$strip)) {
     args$strip <- function(...) strip.default(..., style = 1)
   }
@@ -1639,7 +1683,7 @@ plot.ranef.lme <-
     }
     argData[[".groups"]] <- ordered(argData[[".groups"]],
                                     levels = unique(argData[[".groups"]]))
-    args <- list(formula = argForm, data = argData, ...)
+    args <- list(argForm, data = argData, ...)
     if (is.null(args$xlab)) {
       args$xlab <- attr(object, "label")
     }
@@ -1667,7 +1711,7 @@ plot.ranef.lme <-
     reName <- form[[2]]
     if (length(reName) != 1 &&
         substring(deparse(reName),
-                  nchar(deparse(reName)) - 10) != "(Intercept)") {
+                  nchar(deparse(reName), "c") - 10) != "(Intercept)") {
       stop("Only single effects allowed in left side of form.")
     }
     reName <- deparse(reName)
@@ -1742,9 +1786,9 @@ plot.ranef.lme <-
 
     condvar <- eval(expression(g), argData)
     xscales.lim <- as.list(levels(condvar))
-    subsc <- seq(along = condvar)
+    subsc <- seq_along(condvar)
 
-    for (i in seq(along = xscales.lim)) {
+    for (i in seq_along(xscales.lim)) {
         subscripts <- subsc[condvar == xscales.lim[[i]]]
         vN <- .vNam[subscripts][1]
         if (.vType[vN] == "numeric") {
@@ -1794,7 +1838,8 @@ predict.lme <-
   }
   maxQ <- max(level)			# maximum level for predictions
   mCall <- object$call
-  fixed <- eval(mCall[["fixed"]][-2])
+  fixed <- eval(eval(mCall$fixed)[-2])
+  Terms <- object$terms
   newdata <- as.data.frame(newdata)
 
   if (maxQ > 0) {			# predictions with random effects
@@ -1888,7 +1933,7 @@ predict.lme <-
   N <- nrow(dataMix)
   if (length(all.vars(fixed)) > 0) {
 #    X <- model.matrix(fixed, model.frame(fixed, dataMix), contr)
-    X <- model.matrix(fixed, model.frame(fixed, dataMix))
+    X <- model.matrix(fixed, model.frame(delete.response(Terms), dataMix))
   } else {
     X <- array(1, c(N, 1), list(row.names(dataMix), "(Intercept)"))
   }
@@ -1952,6 +1997,7 @@ predict.lme <-
 print.anova.lme <-
   function(x, verbose = attr(x, "verbose"), ...)
 {
+  ox <- x
   if ((rt <- attr(x,"rt")) == 1) {
     if (!is.null(lab <- attr(x, "label"))) {
       cat(lab)
@@ -1994,8 +2040,9 @@ print.anova.lme <-
       }
       x[[i]] <- xx
     }
-    invisible(print(as.data.frame(x)))
+    print(as.data.frame(x))
   }
+  invisible(ox)
 }
 
 print.intervals.lme <-
@@ -2013,9 +2060,10 @@ print.intervals.lme <-
       }
     } else {
       if (i == "sigma") print(c(aux), ...)
-      else print.matrix(aux, ...)
+      else print(as.matrix(aux), ...)
     }
   }
+  invisible(x)
 }
 
 print.lme <-
@@ -2060,6 +2108,7 @@ print.lme <-
     cat("\n")
     print(rev(Ngrps))
   }
+  invisible(x)
 }
 
 print.ranef.lme <-
@@ -2068,13 +2117,13 @@ print.ranef.lme <-
   if (!inherits(x[[1]], "data.frame")) {
     print.data.frame(x, ...)
   } else {                              # list
-    for(i in seq(along = x)) {
+    for(i in seq_along(x)) {
       cat("Level:", attr(x, "grpNames")[i],"\n")
       print.data.frame(x[[i]])
       if (i < length(x)) cat("\n")
     }
   }
-  invisible(0)
+  invisible(x)
 }
 
 print.summary.lme <-
@@ -2144,6 +2193,7 @@ print.summary.lme <-
     cat("\n")
     print(rev(Ngrps))
   }
+  invisible(x)
 }
 
 qqnorm.lme <-
@@ -2329,7 +2379,7 @@ qqnorm.lme <-
   if (is.null(args$cex)) args$cex <- par("cex")
   if (is.null(args$adj)) args$adj <- par("adj")
 
-  args <- c(list(formula = eval(parse(text = dform)),
+  args <- c(list(eval(parse(text = dform)),
                  data = substitute(data)), args)
   if (is.null(args$panel)) {
     args <- c(list(panel = function(x, y, subscripts, ...){
@@ -2342,7 +2392,7 @@ qqnorm.lme <-
           ltext(x[ids], y[ids], idLabels[subscripts][ids],
                 cex = dots$cex, adj = dots$adj)
       }
-      if (!is.null(abl)) panel.abline(abl, ...)
+      if (!is.null(abl)) { if (length(abl) == 2) panel.abline(a = abl, ...) else panel.abline(h = abl, ...) }
     }), args)
   }
   if(type == "reff" && !std) {
@@ -2471,7 +2521,9 @@ residuals.lme <-
     if (asList) {
       val <- as.list(split(val, ordered(grps, levels = unique(grps))))
     } else {
-      names(val) <- grps
+      grp.nm <- row.names(object[["groups"]])
+      val <-naresid(object$na.action, val)
+      names(val) <- grps[match(names(val), grp.nm)]
     }
     attr(val, "label") <-
       switch(type,
@@ -2485,8 +2537,8 @@ residuals.lme <-
              pearson = "Standardized residuals",
              normalized = "Normalized residuals"
              )
-  }
-  val
+    val
+  } else naresid(object$na.action, val)
 }
 
 summary.lme <- function(object, adjustSigma = TRUE, verbose = FALSE, ...)
@@ -2513,7 +2565,7 @@ summary.lme <- function(object, adjustSigma = TRUE, verbose = FALSE, ...)
   ##
   resd <- resid(object, type = "pearson")
   if (length(resd) > 5) {
-    resd <- quantile(resd)
+    resd <- quantile(resd, na.rm = TRUE) # might have NAs from na.exclude
     names(resd) <- c("Min","Q1","Med","Q3","Max")
   }
   object$residuals <- resd
@@ -2624,7 +2676,7 @@ Variogram.lme <-
       if (length(all.vars(covForm)) > 0) {
         if (attr(terms(covForm), "intercept") == 1) {
           covForm <-
-            eval(parse(text = paste("~", deparse(covForm[[2]]),"-1",sep="")))
+            eval(parse(text = paste("~", c_deparse(covForm[[2]]),"-1",sep="")))
         }
         covar <- model.frame(covForm, data, na.action = na.action)
         ## making sure grps is consistent
@@ -2672,7 +2724,7 @@ Variogram.lme <-
         breaks <- c(breaks, udist[2])
       }
       if (!missing(nint) && nint != (length(breaks) - 1)) {
-        stop("Nint is not consistent with breaks.")
+        stop("'nint' is not consistent with 'breaks'")
       }
       nint <- length(breaks) - 1
     }
@@ -2701,8 +2753,8 @@ Variogram.lme <-
                     data.frame(variog = vrg, dist = dst)
                   }, robust = robust)
     val <- do.call("rbind", val)
-    val <- na.omit(val)                 # getting rid of NAs
     val$n.pairs <- as.vector(table(na.omit(cutDist)))
+    val <- na.omit(val)                 # getting rid of NAs
   }
   row.names(val) <- 1:nrow(val)
   if (inherits(csT, "corSpatial") && resType != "normalized") {
@@ -2769,7 +2821,10 @@ fitted.lmeStruct <-
       (conLin$Xy[, ZXstart[j]:ZXend[j], drop = FALSE] *
        ZXb[[i]][rep(1:ZXngrps[i], ZXlen[[i]]),,drop = FALSE]) %*% rep(1, nc[i])
   }
-  fit[, level + 1]
+  ## this is documented to return a vector for one level, matrix for more.
+  ## So it should be a matrix if there is only one row, but not if
+  ## there is only one columns.
+  if(length(level) > 1) fit[, level + 1, drop = FALSE] else fit[, level+1]
 }
 
 Initialize.lmeStruct <-
@@ -2779,7 +2834,7 @@ Initialize.lmeStruct <-
   object[] <- lapply(object, Initialize, data, conLin, control)
   theta <- lapply(object, coef)
   len <- unlist(lapply(theta, length))
-  num <- seq(along = len)
+  num <- seq_along(len)
   if (sum(len) > 0) {
     pmap <- outer(rep(num, len), num, "==")
   } else {
@@ -2815,14 +2870,13 @@ logLik.lmeStructInt <-
   ## logLik for objects with reStruct parameters only, with
   ## internally defined class
   q <- length(Pars)
-  aux <- .C("mixed_loglik",
+  aux <- .C(mixed_loglik,
 	    as.double(conLin[["Xy"]]),
 	    as.integer(unlist(conLin$dims)),
 	    as.double(Pars),
 	    as.integer(attr(object, "settings")),
 	    val = double(1 + q * (q + 1)),
-	    double(1),
-	    PACKAGE = "nlme")[["val"]]
+	    double(1))[["val"]]
   val <- aux[1]
   attr(val, "gradient") <- -aux[1 + (1:q)]
   attr(val, "hessian") <- -array(aux[-(1:(q+1))], c(q, q))
@@ -2866,18 +2920,19 @@ lmeScale <- function(start)
 
 lmeControl <-
   ## Control parameters for lme
-  function(maxIter = 50, msMaxIter = 50, tolerance = 1e-6, niterEM = 25,
+  function(maxIter = 50, msMaxIter = 50, tolerance = 1e-6, niterEM = 25, msMaxEval = 200,
 	   msTol = 1e-7, msScale = lmeScale, msVerbose = FALSE,
            returnObject = FALSE, gradHess = TRUE, apVar = TRUE,
 	   .relStep = (.Machine$double.eps)^(1/3), minAbsParApVar = 0.05,
-           nlmStepMax = 100.0,
-           natural = TRUE)
+           nlmStepMax = 100.0, opt = c("nlminb", "optim"),
+	   optimMethod = "BFGS", natural = TRUE)
 {
   list(maxIter = maxIter, msMaxIter = msMaxIter, tolerance = tolerance,
-       niterEM = niterEM, msTol = msTol, msScale = msScale,
+       niterEM = niterEM, msMaxEval = msMaxEval, msTol = msTol, msScale = msScale,
        msVerbose = msVerbose, returnObject = returnObject,
        gradHess = gradHess , apVar = apVar, .relStep = .relStep,
-       nlmStepMax = nlmStepMax,
+       nlmStepMax = nlmStepMax, opt = match.arg(opt),
+       optimMethod = optimMethod,
        minAbsParApVar = minAbsParApVar, natural = natural)
 }
 
