@@ -1,3 +1,19 @@
+#  File src/library/graphics/R/hist.R
+#  Part of the R package, http://www.R-project.org
+#
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 2 of the License, or
+#  (at your option) any later version.
+#
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#
+#  A copy of the GNU General Public License is available at
+#  http://www.r-project.org/Licenses/
+
 hist <- function(x, ...) UseMethod("hist")
 
 hist.default <-
@@ -96,7 +112,7 @@ hist.default <-
     if (sum(counts) < n)
 	stop("some 'x' not counted; maybe 'breaks' do not span range of 'x'")
     dens <- counts/(n*h)
-    mids <- 0.5 * (breaks[-1] + breaks[-nB])
+    mids <- 0.5 * (breaks[-1L] + breaks[-nB])
     r <- structure(list(breaks = breaks, counts = counts,
 			intensities = dens,
 			density = dens, mids = mids,
@@ -134,7 +150,7 @@ plot.histogram <-
               sub = NULL,
 	      xlab = x$xname, ylab,
 	      xlim = range(x$breaks), ylim = NULL,
-	      axes = TRUE, labels = FALSE, add = FALSE, ...)
+	      axes = TRUE, labels = FALSE, add = FALSE, ann = TRUE, ...)
 {
     equidist <-
 	if(is.logical(x$equidist)) x$equidist
@@ -155,13 +171,13 @@ plot.histogram <-
 	    ylab <- if (!freq) "Density" else "Frequency"
 	plot.new()
 	plot.window(xlim, ylim, "")	#-> ylim's default from 'y'
-	title(main = main, sub = sub, xlab = xlab, ylab = ylab, ...)
+	if(ann) title(main = main, sub = sub, xlab = xlab, ylab = ylab, ...)
 	if(axes) {
 	    axis(1, ...)
 	    axis(2, ...)
 	}
     }
-    rect(x$breaks[-nB], 0, x$breaks[-1], y,
+    rect(x$breaks[-nB], 0, x$breaks[-1L], y,
 	 col = col, border = border,
 	 angle = angle, density = density, lty = lty)
     if((logl <- is.logical(labels) && labels) || is.character(labels))
