@@ -14,12 +14,10 @@
  *  GNU General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin Street Fifth Floor, Boston, MA 02110-1301  USA
+ *  along with this program; if not, a copy is available at
+ *  http://www.r-project.org/Licenses/
  */
 
-/* <UTF8> char here is either ASCII or handled as a whole */
-
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -27,25 +25,6 @@
 
 #include "Defn.h"
 
-/*  append - append second to the tail of first    */
-/*           This operation is non-destructive     */
-/*           i.e. first and second are duplicated  */
-
-SEXP Rf_append(SEXP first, SEXP second)
-{
-    SEXP e;
-
-    PROTECT(second);
-    first = duplicate(first);
-    UNPROTECT(1);
-    PROTECT(first);
-    second = duplicate(second);
-    UNPROTECT(1);
-    for (e = first; CDR(e) != R_NilValue; e = CDR(e));
-    SETCDR(e, second);
-    return first;
-}
-
 
 /*  mkPRIMSXP - return a builtin function      */
 /*              either "builtin" or "special"  */
@@ -58,8 +37,8 @@ SEXP attribute_hidden mkPRIMSXP(int offset, int eval)
 }
 
 /* This is called by function() {}, where an invalid
-   body should be impossible. When called from 
-   other places (eg do_asfunction) they 
+   body should be impossible. When called from
+   other places (eg do_asfunction) they
    should do this checking in advance */
 
 /*  mkCLOSXP - return a closure with formals f,  */
@@ -77,7 +56,7 @@ SEXP attribute_hidden mkCLOSXP(SEXP formals, SEXP body, SEXP rho)
     if(isList(formals))
 	SET_FORMALS(c, formals);
     else
-        error(_("invalid formal arguments for \"function\""));
+	error(_("invalid formal arguments for \"function\""));
 #else
     SET_FORMALS(c, formals);
 #endif
@@ -89,7 +68,7 @@ SEXP attribute_hidden mkCLOSXP(SEXP formals, SEXP body, SEXP rho)
        )
 	SET_BODY(c, body);
     else
-        error(_("invalid body argument for \"function\"\n\
+	error(_("invalid body argument for \"function\"\n\
 Should NEVER happen; please bug.report() [mkCLOSXP]"));
 
     if(rho == R_NilValue)
@@ -107,13 +86,15 @@ Should NEVER happen; please bug.report() [mkCLOSXP]"));
 
 static int isDDName(SEXP name)
 {
-    char *buf, *endp;
+    const char *buf;
+    char *endp;
+    long val;
 
     buf = CHAR(name);
-    if( !strncmp(buf,"..",2) && strlen(buf) > 2 ) {
-        buf += 2;
-	strtol(buf, &endp, 10);
-        if( *endp != '\0')
+    if( !strncmp(buf, "..", 2) && strlen(buf) > 2 ) {
+	buf += 2;
+	val = strtol(buf, &endp, 10);
+	if( *endp != '\0')
 	    return 0;
 	else
 	    return 1;
