@@ -540,6 +540,15 @@ stopifnot(Inf == qgamma(1, sh))
 stopifnot(0   == qgamma(0, sh))
 ## the first gave Inf, NaN, and 99.425 in R 2.1.1 and earlier
 
+## In extreme left tail {PR#11030}
+qg <- qgamma(10:123*1e-12, shape=19)
+qg2<- qgamma(1:100 * 1e-9, shape=11)
+stopifnot(diff(qg, diff=2) < -6e-6,
+          diff(qg2,diff=2) < -6e-6,
+          All.eq(qg  [1], 2.35047385139143),
+          All.eq(qg2[30], 1.11512318734547))
+## was non-continuous in R 2.6.2 and earlier
+
 f2 <- c(0.5, 1:4)
 stopifnot(df(0, 1, f2) == Inf,
           df(0, 2, f2) == 1,
@@ -552,11 +561,22 @@ stopifnot(pbinom(x0, size = 3, prob = 0.1) == 0,
 ## very small negatives were rounded to 0 in R 2.2.1 and earlier
 
 ## dbeta(*, ncp):
+db.x <- c(0, 5, 80, 405, 1280, 3125, 6480, 12005, 20480, 32805,
+	  50000, 73205, 103680, 142805, 192080, 253125, 327680)
 a <- rlnorm(100)
 stopifnot(All.eq(a, dbeta(0, 1, a, ncp=0)),
-          dbeta(0, 0.9, 2.2, ncp = c(0, a)) == Inf
+	  dbeta(0, 0.9, 2.2, ncp = c(0, a)) == Inf,
+	  All.eq(65536 * dbeta(0:16/16, 5,1), db.x),
+	  All.eq(exp(16 * log(2) + dbeta(0:16/16, 5,1, log=TRUE)), db.x)
+          )
+## the first gave 0, the 2nd NaN in R <= 2.3.0; others use 'TRUE' values
+stopifnot(all.equal(dbeta(0.8, 0.5, 5, ncp=1000),# was way too small in R <= 2.6.2
+		    3.001852308909e-35),
+	  all.equal(1, integrate(dbeta, 0,1, 0.8, 0.5, ncp=1000)$value,
+		    tol=1e-4),
+          all.equal(1, integrate(dbeta, 0,1, 0.5, 200, ncp=720)$value),
+          all.equal(1, integrate(dbeta, 0,1, 125, 200, ncp=2000)$value)
           )
-## the first gave 0, the 2nd NaN in R <= 2.3.0
 
 ## df(*, ncp):
 x <- seq(0, 10, length=101)
@@ -610,6 +630,62 @@ for(nu in df.set) {
     stopifnot(is.finite(pqq))
 }
 
+All.eq(pt(2^-30, df=10),
+       0.50000000036238542)# = .5+ integrate(dt, 0,2^-30, df=10, rel.tol=1e-20)
+
+## rbinom(*, size) gave NaN for large size up to R <= 2.6.1
+M <- .Machine$integer.max
+set.seed(7)
+tt <- table(rbinom(100,    M, pr = 1e-9)) # had values in {0,2} only
+t2 <- table(rbinom(100, 10*M, pr = 1e-10))
+stopifnot(names(tt) == 0:6, sum(tt) == 100, sum(t2) == 100) ## no NaN there
+
+## qf() with large df1, df2  and/or  small p:
+x <- 0.01; f1 <- 1e60; f2 <- 1e90
+stopifnot(qf(1/4, Inf, Inf) == 1,
+	  all.equal(1, 1e-18/ pf(qf(1e-18, 12,50), 12,50), tol=1e-10),
+	  abs(x - qf(pf(x, f1,f2, log.p=TRUE), f1,f2, log.p=TRUE)) < 1e-4)
+
+## qbeta(*, log.p) for "border" case:
+stopifnot(is.finite(qbeta(-1e10, 50,40, log.p=TRUE)),
+          is.finite(qbeta(-1e10,  2, 3, lower=FALSE, log.p=TRUE)))
+## infinite loop or NaN in R <= 2.7.0
+
+## phyper(x, 0,0,0), notably for huge x
+stopifnot(all(phyper(c(0:3, 1e67), 0,0,0) == 1))
+## practically infinite loop and NaN in R <= 2.7.1 (PR#11813)
+
+## plnorm(<= 0, . , log.p=TRUE)
+stopifnot(plnorm(-1:0, lower.tail=FALSE, log.p=TRUE) == 0,
+	  plnorm(-1:0, lower.tail=TRUE,	 log.p=TRUE) == -Inf)
+## was wrongly == 'log.p=FALSE' up to R <= 2.7.1 (PR#11867)
+
+
+## pchisq(df=0) was wrong in 2.7.1
+stopifnot(pchisq(c(-1,0,1), df=0) == c(0,1,1),
+          pchisq(c(-1,0,1), df=0, lower.tail=FALSE) == c(1,0,0))
+
+## dnbinom for extreme  size and/or mu :
+mu <- 20
+d <- dnbinom(17, mu=mu, size = 1e11*2^(1:10)) - dpois(17, lambda=mu)
+stopifnot(d < 0, diff(d) > 0, d[1] < 1e-10)
+## was wrong up to 2.7.1
+## The fix to the above, for x = 0, had a new cancellation problem
+mu <- 1e12 * 2^(0:20)
+stopifnot(all.equal(1/(1+mu), dnbinom(0, size = 1, mu = mu), tol=1e-13))
+## was wrong in 2.7.2 (only)
+
+
+## Non-central F for large x
+x <- 1e16 * 1.1 ^ (0:20)
+dP <- diff(pf(x, df1=1, df2=1, ncp=20, lower.tail=FALSE, log=TRUE))
+stopifnot(-0.047 < dP, dP < -0.0455)
+## pf(*, log) jumped to -Inf prematurely in 2.8.0 and earlier
+
+
+## Non-central Chi^2 density for large x
+stopifnot(0 == dchisq(c(Inf, 1e80, 1e50, 1e40), df=10, ncp=1))
+## did hang in 2.8.0 and earlier (PR#13309).
 
 
 cat("Time elapsed: ", proc.time() - .ptime,"\n")
